package org.gcube.data.analysis.rconnector;

import java.io.File;
import java.net.URI;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.inject.Inject;
import javax.servlet.ServletContext;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.NewCookie;
import javax.ws.rs.core.Response;
import javax.xml.bind.DatatypeConverter;

import org.gcube.common.authorization.library.provider.AuthorizationProvider;
import org.gcube.common.authorization.library.provider.CalledMethodProvider;
import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
import org.gcube.common.scope.api.ScopeProvider;

import lombok.extern.slf4j.Slf4j;

@Path("connect/")
@Slf4j
public class Resource {

	@Inject
	ConfigFileWriter writer;

	@Inject
	InfoRetriever infoRetriever;

	@Context ServletContext context;


	private Response _connect(Long trId) {
		String login = AuthorizationProvider.instance.get().getClient().getId();
		String usersHome = context.getInitParameter("usersHome");
		String filename = context.getInitParameter("filename");
		String configFilePath = context.getInitParameter("userConfigBasePath");
		if (configFilePath==null) configFilePath = usersHome;
		String scriptToExecute = context.getInitParameter("addUserScript");
		if (ScopeProvider.instance.get()==null || login == null ) return Response.serverError().build();

		Info info;
		if (trId!=null) {
			try{
				info = infoRetriever.retrieve(login, trId);
				log.debug("retrieved info: "+info);
			}catch(Exception e){
				log.error("error connecting to r",e);
				return Response.serverError().build();
			}
		} else {
			info= new Info();
			info.setUsername(login);
			info.setToken(SecurityTokenProvider.instance.get());
		}
		if (!writer.write(info, login, filename, usersHome, scriptToExecute)) return Response.serverError().build();

		return createResponse(login);
	}

	@Path("/{trId}")
	@GET
	public Response connect(@PathParam("trId") Long tabularResourceId) {
		CalledMethodProvider.instance.set("connect(TabularResource)");
		log.info("connect called with user {} and trID {} in scope {}",AuthorizationProvider.instance.get().getClient().getId(), tabularResourceId, ScopeProvider.instance.get());
		if (tabularResourceId ==null ) return Response.serverError().build();

		return _connect(tabularResourceId);
	}


	@GET
	public Response connect() {
		CalledMethodProvider.instance.set("connect");
		log.info("connect called with user {} in scope {}",AuthorizationProvider.instance.get().getClient().getId(), ScopeProvider.instance.get());	
		return _connect(null);
	}

	private Response createResponse(String userName){
		try{
			String keyFilePath = context.getInitParameter("storedKeyPath");
			String rStudioServerAddress = context.getInitParameter("rStudioAddress");
			String rStudioProtocol = context.getInitParameter("rStudioProtocol");
			if (rStudioProtocol==null) rStudioProtocol="http";
			
			log.debug("key file path: "+keyFilePath);
			log.debug("rstudio server address: "+rStudioServerAddress);

			Calendar now = Calendar.getInstance();
			now.add(Calendar.YEAR, 10);
			now.add(Calendar.DAY_OF_YEAR, -1);
			SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.ENGLISH);
			String format = sdf.format(now.getTime());

			File keyFile = new File(keyFilePath);
			if (!keyFile.exists())
				return Response.serverError().build();

			byte[] keyByte = Files.readAllBytes(keyFile.toPath());

			SecretKeySpec keySpec = new SecretKeySpec(keyByte,
					"HmacSHA256");
			Mac mac = Mac.getInstance("HmacSHA256");
			mac.init(keySpec);


			byte[] resultUserId = mac.doFinal((userName+format).getBytes());
			String encodeduserId = URLEncoder.encode(DatatypeConverter.printBase64Binary(resultUserId));
			String userIdCookieValue = userName+"|"+URLEncoder.encode(format).replaceAll("\\+", "%20")+"|"+encodeduserId;
			NewCookie userId = new NewCookie("user-id", 
					userIdCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );
			NewCookie userIdLegacy = new NewCookie("user-id-legacy", 
					userIdCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );

			String listUserId = "9c16856330a7400cbbbba228392a5d83";
			byte[] resultUserListId = mac.doFinal((listUserId+format).getBytes());
			String encodeduserListId = URLEncoder.encode(DatatypeConverter.printBase64Binary(resultUserListId));
			String userListIdCookieValue = listUserId+"|"+URLEncoder.encode(format).replaceAll("\\+", "%20")+"|"+encodeduserListId;
			String portTokenCookieValue = "581e7ab0561d";
			
			
			NewCookie userListId = new NewCookie("user-list-id", 
					userListIdCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );

			NewCookie userListIdLegacy = new NewCookie("user-list-id-legacy", 
					userListIdCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );
			
			NewCookie persistAuth = new NewCookie("persist-auth", 
					"0",
					"/", rStudioServerAddress, "", -1, false, true );
			
			NewCookie portToken = new NewCookie("port-token", 
					portTokenCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );
			
			NewCookie portTokenLegacy = new NewCookie("port-token-legacy", 
					portTokenCookieValue,
					"/", rStudioServerAddress, "", -1, false, true );
			

			return Response.seeOther(new URI(String.format("%s://%s", rStudioProtocol, rStudioServerAddress)))
					.cookie(userId, userIdLegacy, userListId, userListIdLegacy,persistAuth, portToken, portTokenLegacy).build();
		}catch(Exception e){
			log.error("error creating response", e);
			return Response.serverError().build();}
	}

}
